@@ -1,4 +1,4 @@
-/* $Id: strformatrt.cpp 114404 2017-04-05 13:02:36Z fmehnert $ */
+/* $Id: strformatrt.cpp 115341 2017-05-10 14:00:11Z fmehnert $ */
 /** @file
  * IPRT - IPRT String Formatter Extensions.
  */
@@ -82,9 +82,9 @@ static size_t rtstrFormatIPv6HexWord(char *pszDst, uint16_t uWord)
     off = 0;
     switch (cDigits)
     {
-        case 4: pszDst[off++] = g_szHexDigits[(uWord >> 12) & 0xf];
-        case 3: pszDst[off++] = g_szHexDigits[(uWord >>  8) & 0xf];
-        case 2: pszDst[off++] = g_szHexDigits[(uWord >>  4) & 0xf];
+        case 4: pszDst[off++] = g_szHexDigits[(uWord >> 12) & 0xf]; /* fall thru */
+        case 3: pszDst[off++] = g_szHexDigits[(uWord >>  8) & 0xf]; /* fall thru */
+        case 2: pszDst[off++] = g_szHexDigits[(uWord >>  4) & 0xf]; /* fall thru */
         case 1: pszDst[off++] = g_szHexDigits[(uWord >>  0) & 0xf];
             break;
     }
